/*
 * Copyright (c) 2008 Hans - Joachim Borchers
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef HWDEF_H_
#define HWDEF_H_

// #define POLLIN_BOARD
#define TARGET_BOARD

/*
 * port pin defintion for ATMEGA16
 * 
 * PA0: Analog In, temperature solder gun 1
 * PA1: Analog In, temperature solder gun 2
 * PA2: Analog In, spare connect to DIN connector
 * PA3: Analog In, spare connect to DIN connector
 * PA4: digital In, push botton 1
 * PA5: digital In, push botton 2
 * PA6: digital In, push botton 3
 * PA7: digital In, push botton 4
 * 
 * PB0: digital I/Os: LCD data port
 * ...  (bit 5-8 via jumper to SPI I/F)!!
 * PB7: 
 *
 * PC0: digital In, encoder signal A
 * PC1: digital In, encoder signal B
 * PC2: JTAG
 * PC3: JTAG
 * PC4: JTAG
 * PC5: JTAG
 * PC6: digital out: trigger TRIAC solder gun 1
 * PC7: digital out: trigger TRIAC solder gun 2
 * 
 * PD0: RS232 Rx
 * PD1: RS232 Tx
 * PD2: digital In, zero crossing detection
 * PD3: digital I/O: encoder push button
 * PD4: digital I/O: LCD RS
 * PD5: digital I/O: LCD RW 
 * PD6: digital I/O: TESTPIN for scope
 * PD7: digital I/O: LCD EN
 * 
 * 
 */

#ifdef TARGET_BOARD
//--- Testpin
#define TEST_PORT       PORTD
#define TEST_PORT_DDR   DDRD
#define TEST_PIN        PIND
#define TEST_BIT_NO     6
#define TEST_BIT_MASK   (1<<TEST_BIT_NO)
#define TEST_BIT      SBIT( TEST_PORT, 6 )


//--- LCD I/F
#define LCD_DDR_PORT    DDRB                    /*!< Lcd data DDR */
#define LCD_DATA_PORT   PORTB                   /*!< Lcd data PORT */
#define LCD_DATA_PIN    PINB                    /*!< Lcd data PORTIN */

#define RW_DDR          DDRD                    /*!< Read / write DDR */
#define RW_PORT         PORTD                   /*!< Read / write PORT */ 
#define RW              PORTD5                  /*!< Read / write BIT */

#define BUSY_PORT       PORTB                   /*!< Busy PORT */
#define BUSY_PIN_PORT   PINB                    /*!< Busy PORTIN */
#define BUSY            PORTB7                  /*!< Busy BIT */

#define RS_DDR          DDRD                    /*!< Data or command DDR */
#define RS_PORT         PORTD                   /*!< Data or command PORT */
#define RS              PORTD4                  /*!< Data or command BIT */

#define ENA_DDR         DDRD                    /*!< Enable DDR */
#define ENA_PORT        PORTD                   /*!< Enable PORT */ 
#define ENA             PORTD7                  /*!< Enable BIT */

//#define ENA2_DDR        DDRF                    /*!< Enable 2 DDR */
//#define ENA2_PORT       PORTF                   /*!< Enable 2 PORT */
//#define ENA2            PORTF3                  /*!< Enable 2 BIT */

//--- Heater IO
#define HEAT_PWM_PORT PORTC
#define HEAT_PWM_DDR  DDRC
#define HEAT_PWM_PIN  PINC
#define HEAT_PWM_GUN1 6       // soldering gun 1
#define HEAT_PWM_GUN2 7       // soldering gun 2

//--- Zero crossing detection IO
#define ZEROCROSS_PORT PORTD
#define ZEROCROSS_DDR  DDRD
#define ZEROCROSS_PIN  PIND
#define ZEROCROSS_IN   2       // input zero crossing signal (INT0)

//--- Rotary encoder IO
#define ROT_ENC_PORT      PORTC
#define ROT_ENC_DDR       DDRC
#define ROT_ENC_PIN       PINC
#define ROT_ENC_DDR_SIGA  SBIT( ROT_ENC_DDR, 0 )       
#define ROT_ENC_DDR_SIGB  SBIT( ROT_ENC_DDR, 1 )
#define ROT_ENC_PU_A      SBIT( ROT_ENC_PORT, 0 )
#define ROT_ENC_PU_B      SBIT( ROT_ENC_PORT, 1 )
#define ROT_ENC_SIGA      SBIT( ROT_ENC_PIN, 0 )       
#define ROT_ENC_SIGB      SBIT( ROT_ENC_PIN, 1 )     

//--- Rotary encoder push button
#define ROT_ENC_PUSH_PORT PORTD
#define ROT_ENC_PUSH_DDR  DDRD
#define ROT_ENC_PUSH_PIN  PIND
#define ROT_ENC_PUSH_BIT  3
#define ROT_ENC_PU_PUSH   SBIT(ROT_ENC_PUSH_PORT, 3)
#define ROT_ENC_DDR_PUSH  SBIT(ROT_ENC_PUSH_DDR, 3)
#define ROT_ENC_PUSH_BTN  SBIT(ROT_ENC_PUSH_PIN, 3)

//--- general push buttons
#define GEN_PUSH_BUTTON_PORT PORTA
#define GEN_PUSH_BUTTON_DDR  DDRA
#define GEN_PUSH_BUTTON_PIN  PINA
#define GEN_PUSH_BT1_BIT     4
#define GEN_PUSH_BT2_BIT     5
#define GEN_PUSH_BT3_BIT     6
#define GEN_PUSH_BT4_BIT     7
#define GEN_PUSH_BTN_MASK     ((1<<GEN_PUSH_BT1_BIT) | (1<<GEN_PUSH_BT2_BIT) | (1<<GEN_PUSH_BT3_BIT) | (1<<GEN_PUSH_BT4_BIT) )

#define GEN_PUSH_PU_BT1      SBIT(GEN_PUSH_BUTTON_PORT, 4)
#define GEN_PUSH_PU_BT2      SBIT(GEN_PUSH_BUTTON_PORT, 5)
#define GEN_PUSH_PU_BT3      SBIT(GEN_PUSH_BUTTON_PORT, 6)
#define GEN_PUSH_PU_BT4      SBIT(GEN_PUSH_BUTTON_PORT, 7)
#define GEN_PUSH_DDR_BT1     SBIT(GEN_PUSH_BUTTON_DDR, 4)
#define GEN_PUSH_DDR_BT2     SBIT(GEN_PUSH_BUTTON_DDR, 5)
#define GEN_PUSH_DDR_BT3     SBIT(GEN_PUSH_BUTTON_DDR, 6)
#define GEN_PUSH_DDR_BT4     SBIT(GEN_PUSH_BUTTON_DDR, 7)
#define GEN_PUSH_BUTTON1     SBIT(GEN_PUSH_BUTTON_PIN, 4)
#define GEN_PUSH_BUTTON2     SBIT(GEN_PUSH_BUTTON_PIN, 5)
#define GEN_PUSH_BUTTON3     SBIT(GEN_PUSH_BUTTON_PIN, 6)
#define GEN_PUSH_BUTTON4     SBIT(GEN_PUSH_BUTTON_PIN, 7)

//--- 
#endif


#ifdef POLLIN_BOARD
//--- LCD I/F
  #ifndef LCD_CTRL_PORT
    // port and pins you will use for control lines
    #define LCD_CTRL_PORT PORTB
    #define LCD_CTRL_DDR  DDRB
    #define LCD_CTRL_RS   2
    #define LCD_CTRL_RW   1
    #define LCD_CTRL_E    0
  #endif
  #ifndef LCD_DATA_POUT
    // port you will use for data lines
    #define LCD_DATA_POUT PORTA
    #define LCD_DATA_PIN  PINA
    #define LCD_DATA_DDR  DDRA
    // access mode you will use (default is 8bit unless 4bit is selected)
    #define LCD_DATA_4BIT
  #endif

//--- Heater IO
#define HEAT_PWM_PORT PORTC
#define HEAT_PWM_DDR  DDRC
#define HEAT_PWM_PIN  PINC
#define HEAT_PWM_GUN1 6       // soldering gun 1
#define HEAT_PWM_GUN2 7       // soldering gun 2

//--- Zero crossing detection IO
#define ZEROCROSS_PORT PORTD
#define ZEROCROSS_DDR  DDRD
#define ZEROCROSS_PIN  PIND
#define ZEROCROSS_IN   2       // input zero crossing signal (INT0)

//--- Rotary encoder IO
#define ROT_ENC_PORT PORTC
#define ROT_ENC_DDR  DDRC
#define ROT_ENC_PIN  PINC
#define ROT_ENC_SIGA 0       
#define ROT_ENC_SIGB 1       
#endif

#endif /*HWDEF_H_*/
