/*
 * Copyright (c) 2008 Hans - Joachim Borchers
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPERS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE DEVELOPERS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "typedef.h"
#include "sysdef.h"
#include "pid.h"
#include "drveeprom.h"


static PID_REGLER Regler[E_SGUN_MAX];  // Variable Regler1, je nach Bedarf erweitern


void PID_InitModule(void)
{
  // Wertung 2+
  //  #define KP 2.5  
  //  #define KI 0.2
  //  #define KD 2.0

  // nach Ziegler  
  //  #define KP 10.4  
  //  #define KI 2.08
  //  #define KD 13.
    
  // Wertung 3  
  //  #define KP 3.5  
  //  #define KI 0.20
  //  #define KD 0.8
    
  
  E_SOLDER_GUNS n;
  for(n=0; n<E_SGUN_MAX;n++)
  {
    Regler[n].Ta=0.5*10;
    Regler[n].esum=0;  
    Regler[n].ealt=0; 
  }
  PID_RefreshKValues();
}

#if 1
u16 PID_Cyclic (E_SOLDER_GUNS uReglernr, u16 x, u16 w)
{    
#define MAXY  100 
#define MAXESUM  200

  PID_REGLER *r = &Regler[uReglernr];
  s32 e;
  s16 y;
  s32 p,i,d;
  
  // aktuelle Regelabweichung bestimmen
  e = (s16)(w - x);          
  // Summe der Regelabweichung aktualisieren
  if( e > -20 && e < 20 )  // nur wenn abweichung nicht zu gro
    r->esum = r->esum + e;
  else
    r->esum = 0;

  // Integralanteil begrenzen
  if( r->esum > MAXESUM ) r->esum = MAXESUM; 
  if( r->esum < -MAXESUM ) r->esum = -MAXESUM;
  
  // Reglergleichung
  p = (r->Kp * e) * 10000L;                     // Faktor 1000000 (kp:100) und *10000
  i = (r->Ki * r->Ta * r->esum)*1000L;          // Faktor 1000000 ( ki:100, ta:10 *1000 )
  d = ((r->Kd*1000L) / r->Ta) * (e - r->ealt);  // Faktor 1000000 ( kd:100, ta:10 *1000 )
  
  y = (p + i + d) / 1000000L; //berechne Stellgre
  
  r->ealt = e; // Regelabweichung fr nchste Abtastung merken
  
  // Stellgre auf 0..1023 begrenzen (10 bit PWM)
  if (y > MAXY) 
    y = MAXY;
  if (y < 1)
    y = 0;
  return y;
}
#endif

#if 0
u16 PID_Cyclic (E_SOLDER_GUNS uReglernr, u16 x, u16 w)
{    
#define MAXY  100 
#define MAXESUM  200.

  PID_REGLER *r = &Regler[uReglernr];
  float e;
  s16 y;
  
  // aktuelle Regelabweichung bestimmen
  e = (float)((s16)(w - x));          
  // Summe der Regelabweichung aktualisieren
  if( e > -20. && e < 20. )
    r->esum = r->esum + e;
  else
    r->esum = 0.;
//  r->esum = r->esum + e;
  // Integralanteil begrenzen
  if( r->esum > MAXESUM ) r->esum = MAXESUM; 
  if( r->esum < -MAXESUM ) r->esum = -MAXESUM;
  
  // Reglergleichung
  //PID->y = (PID->Kp*PID->e)+(PID->I*PID->Ta*PID->esum)+(PID->D*((PID->e-PID->ealt))/PID->Ta);    // Reglergleichung
  y = (r->Kp * e) + (r->Ki * r->Ta * r->esum) + ( (r->Kd / r->Ta) * (e - r->ealt) );      
  r->ealt = e; // Regelabweichung fr nchste Abtastung merken
  
  // Stellgre auf 0..1023 begrenzen (10 bit PWM)
  if (y > MAXY) 
    y = MAXY;
  if (y < 1)
    y = 0;
  return y;
}
#endif
void PID_RefreshKValues( void )
{
  E_SOLDER_GUNS n;
  T_EEPROM *eep = drveeprom_GetParameter()->tEeprom;
  for(n=0; n<E_SGUN_MAX;n++)
  {
    Regler[n].Kp= eep->fKp[n];
    Regler[n].Ki= eep->fKi[n];
    Regler[n].Kd= eep->fKd[n];
  }
}

#if 0

int PID_Cyclic (int x, int w)
{    
  PID_Einstellung *PID = &Regler1;
  #define MAXV  500 
  
  #define KP 25  
  #define KI 5
  #define KD 0
  #define TA 100 
  
  PID->e = w - x;                         // aktuelle Regelabweichung bestimmen
  if ((PID->y < MAXV)&&(PID->y > 0))      // bei bersteuertem stellglied Integration einfrieren
  {                                       // (Anti-Windup)
    PID->esum = PID->esum + PID->e;       // Summe der Regelabweichung aktualisieren  
  }
  //PID->y = (PID->Kp*PID->e)+(PID->I*PID->Ta*PID->esum)+(PID->D*((PID->e-PID->ealt))/PID->Ta);    // Reglergleichung
  PID->y = (KP*PID->e) + ((KI*TA)*PID->esum) + ((KD/TA)*(PID->e-PID->ealt));    // Reglergleichung  
  PID->ealt = PID->e;                     // Regelabweichung fr nchste Abtastung merken
  if (PID->y > MAXV)                      // Stellgre auf 0..1023 begrenzen (10 bit PWM)
  {
    PID->y = MAXV;
  }
  if (PID->y < 1)
  {
    PID->y = 0;
  }
  return PID->y;
}
#endif
